package envoy.client.ui.container;

import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import envoy.client.data.Settings;
import envoy.client.net.Client;
import envoy.client.ui.Theme;
import envoy.client.ui.list.ComponentList;
import envoy.client.ui.list.ComponentListModel;
import envoy.client.ui.renderer.MessageListRenderer;
import envoy.client.ui.renderer.UserComponentListRenderer;
import envoy.data.Message;
import envoy.data.User;

/**
 * This class defines a dialog to choose contacts from.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ContactsChooserDialog.java</strong><br>
 * Created: <strong>15 Mar 2020</strong><br>
 *
 * @author Leon Hofmeister
 * @since Envoy v0.1-beta
 */
public class ContactsChooserDialog extends JDialog {

	private static final long serialVersionUID = -5774558118579032256L;

	private ComponentList<User>	contactList		= new ComponentList<>(new UserComponentListRenderer());
	private JButton				okButton		= new JButton("Ok");
	private JButton				cancelButton	= new JButton("Cancel");

	private final Theme theme = Settings.getInstance().getTheme(Settings.getInstance().getCurrentTheme());

	private final JPanel contentPanel = new JPanel();

	/**
	 * Shows a modal contacts-chooser dialog and blocks until the
	 * dialog is hidden. If the user presses the "OK" button, then
	 * this method hides/disposes the dialog and returns the selected element (has
	 * yet
	 * to be casted back to its original type due to the limitations of Generics in
	 * Java).
	 * If the user presses the "Cancel" button or closes the dialog without
	 * pressing "OK", then this method disposes the dialog and returns an empty
	 * <code>ArrayList</code>.
	 *
	 * @param title   the title of the dialog
	 * @param message the {@link Message} to display on top of the Dialog
	 * @param client  the client whose contacts should be displayed
	 * @return the selected Element (yet has to be casted to the wanted type due to
	 *         the Generics limitations in Java)
	 * @since Envoy v0.1-beta
	 */
	public static List<User> showForwardingDialog(String title, Message message, Client client) {
		ContactsChooserDialog dialog = new ContactsChooserDialog();
		dialog.setTitle(title);
		dialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		dialog.addCancelButtonActionListener(e -> dialog.dispose());
		dialog.getContentPanel()
			.add(new MessageListRenderer(client.getSender().getId()).getListCellComponent(null, message, false), BorderLayout.NORTH);
		List<User> results = new ArrayList<>();
		dialog.addOkButtonActionListener(e -> { results.addAll(dialog.getContactList().getSelection()); dialog.dispose(); });
		ComponentListModel<User> contactListModel = dialog.getContactList().getModel();
		client.getContacts().getContacts().forEach(user -> contactListModel.add(user));
		dialog.setVisible(true);
		dialog.repaint();
		dialog.revalidate();
		return results.size() > 0 ? results : null;
	}

	/**
	 * @since Envoy v0.1-beta
	 */
	private ContactsChooserDialog() {
		contactList.enableMultipleSelection();
		// setBounds(100, 100, 450, 300);
		setModal(true);
		getContentPane().setLayout(new BorderLayout());
		setBackground(theme.getBackgroundColor());
		setForeground(theme.getMessageTextColor());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(new BorderLayout(0, 0));
		contentPanel.add(contactList, BorderLayout.CENTER);
		{
			JPanel buttonPane = new JPanel();
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.setMnemonic(KeyEvent.VK_ENTER);
				okButton.setActionCommand("OK");
				buttonPane.setLayout(new BorderLayout(0, 0));
				buttonPane.add(okButton, BorderLayout.EAST);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton, BorderLayout.WEST);
			}
		}
	}

	/**
	 * @return the underlying {@link ComponentList}
	 * @since Envoy v0.1-beta
	 */
	private ComponentList<User> getContactList() { return contactList; }

	private void addOkButtonActionListener(ActionListener l) { okButton.addActionListener(l); }

	private void addCancelButtonActionListener(ActionListener l) { cancelButton.addActionListener(l); }

	private JPanel getContentPanel() { return contentPanel; }
}
