package envoy.client.ui.list;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.*;

/**
 * Provides a vertical list layout of components provided in a
 * {@link ComponentListModel}. Similar to {@link javax.swing.JList} but capable
 * of rendering {@link JPanel}s.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ComponentList.java</strong><br>
 * Created: <strong>25.01.2020</strong><br>
 *
 * @param <E> the type of object displayed in this list
 * @author Kai S. K. Engelbart
 * @since Envoy v0.3-alpha
 */
public class ComponentList<E> extends JPanel {

	private ComponentListModel<E>			model;
	private ComponentListCellRenderer<E>	renderer;
	private SelectionMode					selectionMode	= SelectionMode.NONE;
	private Set<Integer>					selection		= new HashSet<>();

	private static final long serialVersionUID = 1759644503942876737L;

	public static enum SelectionMode {
		NONE, SINGLE, MULTIPLE
	}

	/**
	 * Initializes a default {@link ComponentList} without a model or a renderer.
	 *
	 * @since Envoy v0.1-beta
	 */
	public ComponentList() { setLayout(new BoxLayout(this, BoxLayout.Y_AXIS)); }

	/**
	 * Creates an instance of {@link ComponentList}.
	 *
	 * @param renderer the list cell renderer used to display elements provided by
	 *                 the {@link ComponentListModel}
	 * @since Envoy v0.3-alpha
	 */
	public ComponentList(ComponentListCellRenderer<E> renderer) {
		this();
		setRenderer(renderer);
	}

	/**
	 * Creates an instance of {@link ComponentList}.
	 *
	 * @param model    the list model providing the list elements to render
	 * @param renderer the list cell renderer used to display elements provided by
	 *                 the {@link ComponentListModel}
	 * @since Envoy v0.3-alpha
	 */
	public ComponentList(ComponentListModel<E> model, ComponentListCellRenderer<E> renderer) {
		this(renderer);
		setModel(model);
	}

	/**
	 * Sets the list model providing the list elements to render. The rendered
	 * components will be synchronized with the contents of the new model or removed
	 * if the new model is {@code null}.
	 *
	 * @param model the list model to set
	 * @since Envoy v0.3-alpha
	 */
	public void setModel(ComponentListModel<E> model) {

		// Remove old model
		if (this.model != null) this.model.setComponentList(null);

		// Synchronize with new model
		this.model = model;
		if (model != null) model.setComponentList(this);
		synchronizeModel();
	}

	/**
	 * Removes all child components and then adds all components representing the
	 * elements of the {@link ComponentListModel}.
	 *
	 * @since Envoy v0.3-alpha
	 */
	public void synchronizeModel() {
		if (model != null && renderer != null) {
			removeAll();
			model.forEach(this::addElement);
			revalidate();
		}
	}

	/**
	 * Selects a list element by index. If the element is already selected, it is
	 * removed from the selection.
	 *
	 * @param index the index of the selected component
	 * @since Envoy v0.1-beta
	 */
	public void selectElement(int index) {
		if (selection.contains(index)) {

			// Remove selection of element at index
			updateElement(index, false);
			selection.remove(index);
		} else {

			// Remove old selection if single selection is enabled
			if (selectionMode == SelectionMode.SINGLE) clearSelection();

			if (selectionMode != SelectionMode.NONE) {

				// Assign new selection
				selection.add(index);

				// Update element
				updateElement(index, true);
			}
		}

		revalidate();
		repaint();
	}

	/**
	 * Removes the current selection.
	 * 
	 * @since Envoy v0.1-alpha
	 */
	public void clearSelection() {
		selection.forEach(i -> updateElement(i, false));
		selection.clear();
	}

	/**
	 * Adds an object to the list by rendering it with the current
	 * {@link ComponentListCellRenderer}.
	 *
	 * @param elem the element to add
	 * @since Envoy v0.3-alpha
	 */
	void addElement(E elem) { addElement(elem, getComponentCount(), false); }

	/**
	 * Adds an object to the list by rendering it with the current
	 * {@link ComponentListRenderer}. If the renderer is {@code null}, no action is
	 * performed.
	 *
	 * @param elem       the element to add
	 * @param index      the index at which to add the element
	 * @param isSelected the selection state of the element
	 * @since Envoy v0.1-beta
	 */
	private void addElement(E elem, int index, boolean isSelected) {
		if (renderer != null) {
			final JComponent component = renderer.getListCellComponent(this, elem, isSelected);
			component.addMouseListener(getSelectionListener(index));
			add(component, index);
		}
	}

	/**
	 * Replaces a list element with a newly rendered instance of its contents.
	 *
	 * @param index      the index of the element to update
	 * @param isSelected the selection state passed to the {@link ListCellRenderer}
	 * @since Envoy v0.1-beta
	 */
	private void updateElement(int index, boolean isSelected) {
		remove(index);
		addElement(model.get(index), index, isSelected);
	}

	/**
	 * @param componentIndex the index of the list component to which the mouse
	 *                       listener will be added
	 * @return a mouse listener calling the
	 *         {@link ComponentList#selectElement(int)} method with the
	 *         component's index when a left click is performed by the user
	 * @since Envoy v0.1-beta
	 */
	private MouseListener getSelectionListener(int componentIndex) {
		return new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) { if (SwingUtilities.isLeftMouseButton(e)) selectElement(componentIndex); }
		};
	}

	/**
	 * @return a set of all selected indices
	 * @since Envoy v0.1-beta
	 */
	public Set<Integer> getSelection() { return selection; }

	/**
	 * @return the model
	 * @since Envoy v0.1-beta
	 */
	public ComponentListModel<E> getModel() { return model; }

	/**
	 * @return the renderer
	 * @since Envoy v0.1-beta
	 */
	public ComponentListCellRenderer<E> getRenderer() { return renderer; }

	/**
	 * @param renderer the renderer to set
	 * @since Envoy v0.1-beta
	 */
	public void setRenderer(ComponentListCellRenderer<E> renderer) { this.renderer = renderer; }

	/**
	 * @return the selectionMode
	 * @since Envoy v0.1-beta
	 */
	public SelectionMode getSelectionMode() { return selectionMode; }

	/**
	 * @param selectionMode the selectionMode to set
	 * @since Envoy v0.1-beta
	 */
	public void setSelectionMode(SelectionMode selectionMode) { this.selectionMode = selectionMode; }
}
