package envoy.client.ui.renderer;

import java.awt.*;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.EnumMap;

import javax.swing.*;

import envoy.client.data.Settings;
import envoy.client.ui.Color;
import envoy.client.ui.IconUtil;
import envoy.client.ui.list.ComponentList;
import envoy.client.ui.list.ComponentListCellRenderer;
import envoy.data.Message;
import envoy.data.Message.MessageStatus;

/**
 * Defines how a message is displayed.<br>
 * <br>
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>MessageListRenderer.java</strong><br>
 * Created: <strong>19 Oct 2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @author Leon Hofmeister
 * @since Envoy v0.1-alpha
 */
public class MessageListRenderer implements ComponentListCellRenderer<Message> {

	private static EnumMap<MessageStatus, ImageIcon>	statusIcons;
	private static ImageIcon							forwardIcon;

	static {
		try {
			statusIcons = IconUtil.loadByEnum(MessageStatus.class, 16);
			forwardIcon	= IconUtil.load("/icons/forward.png", 16);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private final long senderId;

	/**
	 * Initializes a message list renderer. Messages with the given sender ID will
	 * be aligned on the right side, while all other messages will be aligned on
	 * the left side
	 * 
	 * @param senderId the sender ID of the messages to align on the right side
	 * @since Envoy v0.1-beta
	 */
	public MessageListRenderer(long senderId) { this.senderId = senderId; }

	// TODO: Handle message attachments

	@Override
	public JPanel getListCellComponent(ComponentList<? extends Message> list, Message message, boolean isSelected) {
		final var theme = Settings.getInstance().getThemes().get(Settings.getInstance().getCurrentTheme());

		// Panel
		final var	panel	= new JPanel();
		final int	padding	= (int) (list.getWidth() * 0.35);

		GridBagLayout gbl_panel = new GridBagLayout();
		gbl_panel.columnWidths	= new int[] { 1, 1 };
		gbl_panel.rowHeights	= new int[] { 1, 1 };
		gbl_panel.columnWeights	= new double[] { 1, 1 };
		gbl_panel.rowWeights	= new double[] { 1, 1 };

		panel.setLayout(gbl_panel);
		panel.setBackground(isSelected ? theme.getSelectionColor() : theme.getCellColor());

		// Date Label - The Label that displays the creation date of a message
		var dateLabel = new JLabel(new SimpleDateFormat("dd.MM.yyyy HH:mm").format(message.getCreationDate()));
		dateLabel.setForeground(theme.getDateColor());
		dateLabel.setAlignmentX(1f);
		dateLabel.setFont(new Font("Arial", Font.PLAIN, 12));
		dateLabel.setPreferredSize(dateLabel.getPreferredSize());

		var gbc_dateLabel = new GridBagConstraints();
		gbc_dateLabel.fill	= GridBagConstraints.BOTH;
		gbc_dateLabel.gridx	= 0;
		gbc_dateLabel.gridy	= 0;
		panel.add(dateLabel, gbc_dateLabel);

		// Message area - The JTextArea that displays the text content of a message.
		var messageTextArea = new JTextArea(message.getText());
		messageTextArea.setLineWrap(true);
		messageTextArea.setWrapStyleWord(true);
		messageTextArea.setForeground(theme.getMessageTextColor());
		messageTextArea.setAlignmentX(0.5f);
		messageTextArea.setBackground(theme.getCellColor());
		messageTextArea.setEditable(false);
		var font = new Font("Arial", Font.PLAIN, 14);
		messageTextArea.setFont(font);
		messageTextArea.setSize(list.getMaximumSize().width - padding - 16, 10);

		var gbc_messageTextArea = new GridBagConstraints();
		gbc_messageTextArea.fill	= GridBagConstraints.HORIZONTAL;
		gbc_messageTextArea.gridx	= 0;
		gbc_messageTextArea.gridy	= 1;
		panel.add(messageTextArea, gbc_messageTextArea);

		// Status Label - displays the status of the message
		var statusLabel = new JLabel(statusIcons.get(message.getStatus()));

		var gbc_statusLabel = new GridBagConstraints();
		gbc_statusLabel.gridx	= 1;
		gbc_statusLabel.gridy	= 1;
		panel.add(statusLabel, gbc_statusLabel);

		// Forwarding
		if (message.isForwarded()) {
			var forwardLabel = new JLabel("Forwarded", forwardIcon, SwingConstants.CENTER);
			forwardLabel.setBackground(panel.getBackground());
			forwardLabel.setForeground(Color.lightGray);

			var gbc_forwardLabel = new GridBagConstraints();
			gbc_forwardLabel.fill	= GridBagConstraints.BOTH;
			gbc_forwardLabel.gridx	= 1;
			gbc_forwardLabel.gridy	= 0;
			panel.add(forwardLabel, gbc_forwardLabel);
		}

		// Define an etched border and some space to the messages below
		var ours = senderId == message.getSenderId();
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(0, ours ? padding : 0, 10, ours ? 0 : padding),
				BorderFactory.createEtchedBorder()));

		var size = new Dimension(list.getMaximumSize().width - 50, panel.getPreferredSize().height);

		panel.setPreferredSize(size);
		panel.setMinimumSize(size);
		panel.setMaximumSize(size);

		return panel;
	}
}
